/*
 * flaregen.c
 *
 * Copyright (C) 2021 bzt (bztsrc@gitlab)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * @brief A very simple templater to generate flare item definitons
 *
 */
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define JSON_MAXPATHLEVEL 32

/**
 * jsonstr: zero terminated UTF-8 string
 * key: path to a value, like "0.6.0.1" or "result.items.0.name"
 * returns the value in a malloc'd string or NULL
 */
char *json_get(const char *jsonstr, char *key)
{
    char *m=key, *ret=NULL;
    unsigned char *c=(unsigned char*)jsonstr,*k[JSON_MAXPATHLEVEL+1];
    unsigned char *v,*s=NULL,*e=NULL;
    int l=0, j=1, i, n, x[JSON_MAXPATHLEVEL+1]={0};
    if(!jsonstr || !jsonstr[0] || !key) return NULL;
    while(1) {
        while(*c && *c<=' ') c++;
        if(j) { j=0; k[l]=v=c; }
        switch(*c) {
        case '\"': c++; while(*c && *c!='\"'){if(*c=='\\'){c++;} c++;} break;
        case ':': c++; while(*c && *c<=' ') { c++; } v=c--; break;
        case 0: case ',': case '{': case '[': case '}': case ']':
            if(*v!=','&&*v!='{'&&*v!='['&&*v!=']'&&*v!='}') {
                n=k[0][0]=='{'?1:0;
                m=key;
                for(i=n;i<=l && *m;i++) {
                    if(i!=n) { if(*m=='.') m++; else break; }
                    if(k[i][0]=='\"' && k[i]!=v && (*m<'0'||*m>'9')) {
                        s=e=k[i]+1;
                        while(*e&&*e!='\"') { if(*e=='\\') { e++; } e++; }
                        if(memcmp(m,s,e-s)) break;
                        m+=e-s;
                    } else {
                        if(atoi(m)!=x[i]) break;
                        while(*m && *m!='.') m++;
                    }
                }
                if(!*m) {
                    switch(*v) {
                        case '\"':
                            s=e=v+1;
                            while(*e&&*e!='\"') { if(*e=='\\') { e++; } e++; }
                            break;
                        case 'n': return NULL;
                        default:
                            s=e=v;
                            while(*e&&*e>' '&&*e!=','&&*e!=']'&&*e!='}') e++;
                            break;
                    }
                    goto end;
                }
            }
            switch(*c) {
            case 0: return NULL;
            case '{': case '[': x[++l]=0; if(l>=(int)sizeof(x)-1) return NULL; break;
            case '}': case ']': l--; break;
            case ',': x[l]++; break;
            }
            j++;
            break;
        }
        c++;
    }
end:if(!*m && e>s) {
        ret=malloc(e-s+1 < 256 ? 256 : e-s+1);
        if(ret) { memset(ret,0,256); memcpy(ret,s,e-s); ret[e-s]=0; }
    }
    return ret;
}

/**
 * The main function
 */
int main(int argc, char **argv)
{
    FILE *f;
    int len, i, n, v, start, end, add, rndmin, rndmax;
    char *json, key[256], *name, *val, *tmp;

    /* get arguments */
    if(argc < 4) {
        printf("Usage:\r\n  flaregen <json template> <start id> <last id>\r\n");
        return 1;
    }

    start = atoi(argv[2]);
    end = atoi(argv[3]);
    if(end < start) {
        fprintf(f, "end id is smaller than start id\r\n");
        return 1;
    }

    f = fopen(argv[1], "rb");
    if(f) {
        fseek(f, 0, SEEK_END);
        len = (unsigned int)ftell(f);
        fseek(f, 0, SEEK_SET);
        json = (char*)malloc(len + 1);
        if(!json) {
            fprintf(f, "memory allocation error\r\n");
            return 2;
        }
        memset(json, 0, len + 1);
        fread(json, len, 1, f);
        fclose(f);
        if(json[0] != '{') {
            fprintf(stderr, "not a valid json %s\r\n", argv[1]);
            return 2;
        }
    } else {
        fprintf(stderr, "unable to open %s\r\n", argv[1]);
        return 2;
    }

    /* initialize variables */
    memset(key, 0, sizeof(key));
    srand(time(NULL));

    /* iterate on ids */
    for(n = start; n <= end; n++) {
        printf("[item]\r\nid=%u\r\n", n);
        /* iterate on fields */
        for(i = 0; i < 256; i++) {
            sprintf(key, "%d.name", i);
            name = json_get(json, key);
            sprintf(key, "%d.value", i);
            val = json_get(json, key);
            if(!name || !val) break;
            if(!strcmp(name, "INCLUDE"))
                printf("INCLUDE ");
            else
                printf("%s=", name);
            free(name);
            add = rndmin = rndmax = 0;
            sprintf(key, "%d.add", i);
            tmp = json_get(json, key);
            if(tmp) { add = atoi(tmp); free(tmp); }
            sprintf(key, "%d.min", i);
            tmp = json_get(json, key);
            if(tmp) { rndmin = atoi(tmp); free(tmp); }
            sprintf(key, "%d.max", i);
            tmp = json_get(json, key);
            if(tmp) { rndmax = atoi(tmp); free(tmp); }
            printf(val, (rndmin && rndmin <= rndmax ? (rand() % (rndmax - rndmin + 1)) + rndmin : n - start + 1) + add);
            printf("\r\n");
            free(val);
        }
        printf("\r\n");
    }
    return 0;
}
